==> gnomemeeting-0.96.0.tar.bz2: gnomemeeting-0.96.0/src/urlhandler.cpp <==

/* GnomeMeeting -- A Video-Conferencing application
 * Copyright (C) 2000-2002 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 *                          urlhandler.cpp  -  description
 *                         -------------------------------
 *   begin                : Sat Jun 8 2002
 *   copyright            : (C) 2000-2002 by Damien Sandras
 *   description          : Multithreaded class to call a given URL.
 *   email                : dsandras@seconix.com
 *
 */


#include "../config.h" 


#include "urlhandler.h"
#include "gnomemeeting.h"
#include "main_window.h"
#include "misc.h"
#include "endpoint.h"
#include "toolbar.h"
#include "ils.h"
#include "menu.h"


#ifndef DISABLE_GNOME
#include <gnome.h>
#endif


/* Declarations */
extern GnomeMeeting *MyApp;
extern GtkWidget *gm;


/* The class */
GMURLHandler::GMURLHandler (PString c)
  :PThread (1000, AutoDeleteThread)
{
  gw = gnomemeeting_get_main_window (gm);
  url = c;
 
  this->Resume ();
}


GMURLHandler::~GMURLHandler ()
{
  quit_mutex.Wait ();

  /* Nothing to do here except waiting that LDAP at worse
     cancels the search */

  quit_mutex.Signal ();
}


void GMURLHandler::Main ()
{
  PString call_address;
  GMILSClient *ils_client = NULL;
  PINDEX slash;
  PINDEX callto;
  PString tmp_url;
  PString ils_server;
  PString ils_port = PString ("389");
  PString mail;
  PString current_call_token;
  gchar *ip = NULL;
  gchar *msg = NULL;
  GmWindow *gw = NULL;
  GMH323EndPoint *endpoint = NULL;
  H323Connection *con = NULL;
  GConfClient *client = gconf_client_get_default ();
  
  gnomemeeting_threads_enter ();
  gw = gnomemeeting_get_main_window (gm);
  gnomemeeting_threads_leave ();

  endpoint = MyApp->Endpoint ();

  
  /* We first remove the callto: part if any */
  tmp_url = url.ToLower ();
  callto = tmp_url.FindLast ("//");
  if (callto != P_MAX_INDEX)
    url = url.Mid (callto + 2, P_MAX_INDEX);

  call_address = url;

  slash = url.Find ('/');
  
  /* We have a callto URL of the form : ils_server(:port)/mail */
  if (slash != P_MAX_INDEX) {

    quit_mutex.Wait ();
    /* We disable the connect button and the connect menu 
       while searching */
    gnomemeeting_threads_enter ();
    gnomemeeting_call_menu_connect_set_sensitive (0, FALSE);
    gnomemeeting_call_menu_connect_set_sensitive (1, FALSE);
    gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), FALSE);
    gnomemeeting_threads_leave ();
  
    ils_server = url.Left (slash);
    PINDEX p = url.Find (':');

    gnomemeeting_threads_enter ();
    gnomemeeting_statusbar_flash (gw->statusbar, _("Searching for user"));
    gnomemeeting_threads_leave ();
    /* There is a port */
    if (p != P_MAX_INDEX) {
    
      ils_port = ils_server.Mid (p + 1, P_MAX_INDEX);
      ils_server = ils_server.Left (p);
    }

    mail = url.Mid (slash + 1, P_MAX_INDEX);

    /* Search for the user on ILS */
    ils_client = GM_ILS_CLIENT (endpoint->GetILSClientThread ());
    
    ip =
      ils_client->Search ((gchar *) (const char *) ils_server, 
			  (gchar *) (const char *) ils_port, 
			  (gchar *) (const char *) mail);


    if (ip == NULL) {

      gnomemeeting_threads_enter ();
      gnomemeeting_statusbar_flash (gw->statusbar, 
				    _("Error while connecting to ILS directory"));
      gnomemeeting_log_insert (gw->history_text_view, 
			       _("Error while connecting to ILS directory"));
      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 0);
      gnomemeeting_call_menu_connect_set_sensitive (0, TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), TRUE);

      if (gw->progress_timeout) {

	gtk_timeout_remove (gw->progress_timeout);
	gw->progress_timeout = 0;
	gtk_widget_hide (GTK_WIDGET (gw->progressbar));
      }

      gnomemeeting_threads_leave ();

      quit_mutex.Signal ();
      return;
    }
     
    if ((ip)&&(!strcmp (ip, "0.0.0.0:1720"))) {

      gnomemeeting_threads_enter ();
      gnomemeeting_statusbar_flash (gw->statusbar, _("User is offline"));
      gnomemeeting_log_insert (gw->history_text_view, _("User is offline"));
      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 0);
      gnomemeeting_call_menu_connect_set_sensitive (0, TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), TRUE);

      if (gw->progress_timeout) {

	gtk_timeout_remove (gw->progress_timeout);
	gw->progress_timeout = 0;
	gtk_widget_hide (GTK_WIDGET (gw->progressbar));
      }

      gnomemeeting_threads_leave ();

      quit_mutex.Signal ();
      return;
    }
    else {

      call_address = PString (ip);
    }

    gnomemeeting_threads_enter ();
    gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), TRUE);
    gnomemeeting_threads_leave ();

    quit_mutex.Signal ();
  }
    

  /* If the user is using MicroTelco, but G.723.1 is not available for
     a reason or another, we add the MicroTelco prefix if the URL seems
     to be a phone number */
  if (gconf_client_get_bool (client, SERVICES_KEY "enable_microtelco", 0)) {

    if (call_address.FindRegEx ("[A-Z][a-z]") == P_MAX_INDEX &&
	call_address.Find (".") == P_MAX_INDEX &&
	endpoint->GetCapabilities ().FindCapability ("G.723.1") == NULL)
      call_address = PString ("0610#") + call_address;
  }

  
  /* Connect to the URL */
  con = 
    endpoint->MakeCallLocked (call_address, current_call_token);
  endpoint->SetCurrentConnection (con);
  endpoint->SetCurrentCallToken (current_call_token);
  endpoint->SetCallingState (1);
  con->Unlock ();
  
  gnomemeeting_threads_enter ();
  gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);


  /* Enable disconnect: we must be able to stop calling */
  gnomemeeting_call_menu_connect_set_sensitive (1, TRUE);
  
  msg = g_strdup_printf (_("Calling %s"), 
			 (const char *) call_address);
  gnomemeeting_log_insert (gw->history_text_view, msg);
  gnomemeeting_log_insert (gw->calls_history_text_view, msg);
  gnomemeeting_statusbar_push (gw->statusbar, msg);
  connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 1);
  g_free (msg);		
  gnomemeeting_threads_leave ();		 
}

==> gnomemeeting-0.96.1.tar.bz2: gnomemeeting-0.96.1/src/urlhandler.cpp <==

/* GnomeMeeting -- A Video-Conferencing application
 * Copyright (C) 2000-2002 Damien Sandras
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

/*
 *                          urlhandler.cpp  -  description
 *                         -------------------------------
 *   begin                : Sat Jun 8 2002
 *   copyright            : (C) 2000-2002 by Damien Sandras
 *   description          : Multithreaded class to call a given URL or to
 *                          answer a call.
 *   email                : dsandras@seconix.com
 *
 */


#include "../config.h" 


#include "urlhandler.h"
#include "gnomemeeting.h"
#include "main_window.h"
#include "misc.h"
#include "endpoint.h"
#include "toolbar.h"
#include "ils.h"
#include "menu.h"


#ifndef DISABLE_GNOME
#include <gnome.h>
#endif


/* Declarations */
extern GnomeMeeting *MyApp;
extern GtkWidget *gm;


/* The class */
GMURLHandler::GMURLHandler (PString c)
  :PThread (1000, AutoDeleteThread)
{
  gw = gnomemeeting_get_main_window (gm);
  url = c;

  answer_call = FALSE;
  
  this->Resume ();
}


GMURLHandler::GMURLHandler ()
  :PThread (1000, AutoDeleteThread)
{
  gw = gnomemeeting_get_main_window (gm);

  answer_call = TRUE;
  
  this->Resume ();
}


GMURLHandler::~GMURLHandler ()
{
  quit_mutex.Wait ();

  /* Nothing to do here except waiting that LDAP at worse
     cancels the search */

  quit_mutex.Signal ();
}


void GMURLHandler::Main ()
{
  PString call_address;
  GMILSClient *ils_client = NULL;
  PINDEX slash;
  PINDEX callto;
  PString tmp_url;
  PString ils_server;
  PString ils_port = PString ("389");
  PString mail;
  PString current_call_token;
  gchar *ip = NULL;
  gchar *msg = NULL;
  GmWindow *gw = NULL;
  GMH323EndPoint *endpoint = NULL;
  H323Connection *con = NULL;
  GConfClient *client = NULL;
  
  gnomemeeting_threads_enter ();
  client = gconf_client_get_default ();
  gw = gnomemeeting_get_main_window (gm);
  gnomemeeting_threads_leave ();

  endpoint = MyApp->Endpoint ();


  if (answer_call && endpoint) {

    con = endpoint->GetCurrentConnection ();

    if (con)
      con->AnsweringCall (H323Connection::AnswerCallNow);

    return;
  }
  
  
  /* We first remove the callto: part if any */
  tmp_url = url.ToLower ();
  callto = tmp_url.FindLast ("//");
  if (callto != P_MAX_INDEX)
    url = url.Mid (callto + 2, P_MAX_INDEX);

  call_address = url;

  slash = url.Find ('/');
  
  /* We have a callto URL of the form : ils_server(:port)/mail */
  if (slash != P_MAX_INDEX) {

    quit_mutex.Wait ();
    /* We disable the connect button and the connect menu 
       while searching */
    gnomemeeting_threads_enter ();
    gnomemeeting_call_menu_connect_set_sensitive (0, FALSE);
    gnomemeeting_call_menu_connect_set_sensitive (1, FALSE);
    gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), FALSE);
    gnomemeeting_threads_leave ();
  
    ils_server = url.Left (slash);
    PINDEX p = url.Find (':');

    gnomemeeting_threads_enter ();
    gnomemeeting_statusbar_flash (gw->statusbar, _("Searching for user"));
    gnomemeeting_threads_leave ();
    /* There is a port */
    if (p != P_MAX_INDEX) {
    
      ils_port = ils_server.Mid (p + 1, P_MAX_INDEX);
      ils_server = ils_server.Left (p);
    }

    mail = url.Mid (slash + 1, P_MAX_INDEX);

    /* Search for the user on ILS */
    ils_client = GM_ILS_CLIENT (endpoint->GetILSClientThread ());
    
    ip =
      ils_client->Search ((gchar *) (const char *) ils_server, 
			  (gchar *) (const char *) ils_port, 
			  (gchar *) (const char *) mail);


    if (ip == NULL) {

      gnomemeeting_threads_enter ();
      gnomemeeting_statusbar_flash (gw->statusbar, 
				    _("Error while connecting to ILS directory"));
      gnomemeeting_log_insert (gw->history_text_view, 
			       _("Error while connecting to ILS directory"));
      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 0);
      gnomemeeting_call_menu_connect_set_sensitive (0, TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), TRUE);

      if (gw->progress_timeout) {

	gtk_timeout_remove (gw->progress_timeout);
	gw->progress_timeout = 0;
	gtk_widget_hide (GTK_WIDGET (gw->progressbar));
      }

      gnomemeeting_threads_leave ();

      quit_mutex.Signal ();
      return;
    }
     
    if ((ip)&&(!strcmp (ip, "0.0.0.0:1720"))) {

      gnomemeeting_threads_enter ();
      gnomemeeting_statusbar_flash (gw->statusbar, _("User is offline"));
      gnomemeeting_log_insert (gw->history_text_view, _("User is offline"));
      connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 0);
      gnomemeeting_call_menu_connect_set_sensitive (0, TRUE);
      gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), TRUE);

      if (gw->progress_timeout) {

	gtk_timeout_remove (gw->progress_timeout);
	gw->progress_timeout = 0;
	gtk_widget_hide (GTK_WIDGET (gw->progressbar));
      }

      gnomemeeting_threads_leave ();

      quit_mutex.Signal ();
      return;
    }
    else {

      call_address = PString (ip);
    }

    gnomemeeting_threads_enter ();
    gtk_widget_set_sensitive (GTK_WIDGET (gw->connect_button), TRUE);
    gnomemeeting_threads_leave ();

    quit_mutex.Signal ();
  }
    

  /* If the user is using MicroTelco, but G.723.1 is not available for
     a reason or another, we add the MicroTelco prefix if the URL seems
     to be a phone number */
  gnomemeeting_threads_enter ();
  if (gconf_client_get_bool (client, SERVICES_KEY "enable_microtelco", 0)) {

    if (call_address.FindRegEx ("[A-Z][a-z]") == P_MAX_INDEX &&
	call_address.Find (".") == P_MAX_INDEX &&
	endpoint->GetCapabilities ().FindCapability ("G.723.1") == NULL)
      call_address = PString ("0610#") + call_address;
  }
  gnomemeeting_threads_leave ();

  
  /* Connect to the URL */
  con = 
    endpoint->MakeCallLocked (call_address, current_call_token);
  endpoint->SetCurrentConnection (con);
  endpoint->SetCurrentCallToken (current_call_token);
  endpoint->SetCallingState (1);
  con->Unlock ();
  
  gnomemeeting_threads_enter ();
  gtk_widget_set_sensitive (GTK_WIDGET (gw->preview_button), FALSE);


  /* Enable disconnect: we must be able to stop calling */
  gnomemeeting_call_menu_connect_set_sensitive (1, TRUE);
  
  msg = g_strdup_printf (_("Calling %s"), 
			 (const char *) call_address);
  gnomemeeting_log_insert (gw->history_text_view, msg);
  gnomemeeting_log_insert (gw->calls_history_text_view, msg);
  gnomemeeting_statusbar_push (gw->statusbar, msg);
  connect_button_update_pixmap (GTK_TOGGLE_BUTTON (gw->connect_button), 1);
  g_free (msg);		
  gnomemeeting_threads_leave ();		 
}
